<?php
/*------------------------------------------------------------------------
# mod_authorlist - Author List Module
# ------------------------------------------------------------------------
# author    Jesus Vargas
# copyright Copyright (C) 2010 www.munditico.com. All Rights Reserved.
# @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
# Websites: http://www.munditico.com
# Technical Support:  Forum - http://www.munditico.com/forum
-------------------------------------------------------------------------*/

defined('_JEXEC') or die;

require_once JPATH_SITE.'/components/com_authorlist/helpers/route.php';

JModelLegacy::addIncludePath(JPATH_SITE.'/components/com_authorlist/models', 'AuthorListModel');

abstract class modAuthorListHelper
{
	public static function getList(&$params)
	{
		$model = JModelLegacy::getInstance('Authors', 'AuthorListModel', array('ignore_request' => true));

		$app = JFactory::getApplication();
		$appParams = $app->getParams();
		$model->setState('params', $appParams);

		$model->setState('list.start', 0);
		$model->setState('list.limit', (int) $params->get('count', ''));
		$model->setState('filter.published', 1);

		$access = !JComponentHelper::getParams('com_authorlist')->get('show_noauth');
		$authorised = JAccess::getAuthorisedViewLevels(JFactory::getUser()->get('id'));
		$model->setState('filter.access', $access);

		$model->setState('filter.language', $app->getLanguageFilter());
		
		$ordering    = $params->get('ordering', 'order');
		$show_name   = $params->get('show_name', 'name');
		$use_alias   = $params->get('use_alias', 0);
		$author_ids  = $params->get('author_ids', '');
		$author_articles = $params->get('count_articles', 0);		
		$ids         = array();
		if ($author_ids) {
			$ids = array_map('trim', explode(',',$author_ids));
			$filter_type = $params->get('filter_type', 0);
		}
		$model->setState('filter.group_id', $params->get('gid', 0));
		
		$orderCol  = 'a.ordering';
		$orderDir  = 'ASC';
		
		if ( $ordering != 'order' ) :	
			if ( $ordering == 'rand' ) :
				$orderCol  = 'RAND()';	
				$orderDir  = '';
			else :
				$orderCol  = $show_name;
				$orderDir  = strtoupper($ordering);
			endif;
		endif;

		$model->setState('list.ordering', $orderCol);
		$model->setState('list.direction', $orderDir);

		$items = $model->getItems();
				
		$i = -1;
		foreach ($items as &$item)
		{
			$i++;
			if (count($ids))  {
				if (in_array($item->id, $ids) != $filter_type) {
					unset($items[$i]);
					continue;	
				}
			}
			$item->slug = ($item->alias?$item->id.':'.$item->alias:$item->id);
			if ($item->display_alias) {
				$item->name = $item->display_alias;
				$item->slug = $item->id.':'.JApplication::stringURLSafe($item->display_alias);		
			}
			elseif ($show_name != 'name')
			{
				$item->name = $item->username;	
			}
			if ($access || in_array($item->access, $authorised))
			{
				$item->link = JRoute::_(AuthorListHelperRoute::getAuthorRoute($item->slug));
			} else {
				$item->link = JRoute::_('index.php?option=com_users&view=login');
			}
			
			if ($params->get('show_image') && ($item->image or $params->get('def_image')))
			{
				$item->image = modAuthorListHelper::getThumbnail($item,$params);
			}
			if ((int)$author_articles)
			{
				$item->articles = modAuthorListHelper::getArticles($item->userid, $params);
			}
		}

		return $items;
	}
	
	public function getArticles($author_id, &$params) { 
  
  		$model = JModelLegacy::getInstance('Articles', 'AuthorListModel', array('ignore_request' => true));
		$model->setState('filter.published', 1);
  		$model->setState('filter.author_id', $author_id);
  		$model->setState('params', JComponentHelper::getParams('com_authorlist', true));
		$model->setState('list.limit', $params->get('count_articles'));
		
		$queryDate = $params->get('order_date','created');
		switch ($params->get('orderby','rdate'))
		{
			case 'date' :
				$orderby  = $queryDate;
				$orderdir = '';
				break;

			case 'rdate' :
				$orderby  = $queryDate;
				$orderdir = 'DESC';
				break;

			case 'alpha' :
				$orderby  = 'a.title';
				$orderdir = '';
				break;

			case 'ralpha' :
				$orderby  = 'a.title';
				$orderdir = 'DESC';
				break;

			case 'hits' :
				$orderby = 'a.hits';
				$orderdir = 'DESC';
				break;

			case 'rhits' :
				$orderby = 'a.hits';
				$orderdir = '';
				break;

			case 'order' :
				$orderby = 'a.ordering';
				$orderdir = '';
				break;

			case 'author' :
				$orderby = 'author';
				$orderdir = '';
				break;

			case 'rauthor' :
				$orderby = 'author';
				$orderdir = 'DESC';
				break;

			case 'front' :
				$orderby = 'fp.ordering';
				$orderdir = '';
				break;

			case 'rand' :
				$orderby = 'RAND()';
				$orderdir = '';
				break;

			default :
				$orderby = 'a.ordering';
				$orderdir = '';
				break;
		}
		$model->setState('list.ordering', $orderby);
		$model->setState('list.direction', $orderdir);
				
		$catids = $params->get('catid');
		$model->setState('filter.category_id.include', (bool) $params->get('category_filtering_type', 1));
		
		if ($catids)
		{		
			$model->setState('filter.category_id', $catids);	
		}
		
      	return $model->getItems();
  	}  
	
	public static function getThumbnail($item,&$params) 
	{
		
		$width      = $params->get('thumb_width',30);
		$height     = $params->get('thumb_height',30);
		$proportion = $params->get('thumb_proportions','bestfit');
		$img_type   = $params->get('thumb_type','');
		$bgcolor    = hexdec($params->get('thumb_bg','#FFFFFF'));
		
		$def_image  = $params->get('def_image', '');
		$author_img = $item->image?$item->image:$def_image;
		
		$img_name   = pathinfo($author_img, PATHINFO_FILENAME);
		$img_ext    = pathinfo($author_img, PATHINFO_EXTENSION);
		$img_path   = JPATH_BASE  . '/' . $author_img;		
		$img_base   = str_replace('modules/mod_authorlist/assets/','',JURI::base(false));
		
		$size = @getimagesize($img_path);
		
		$errors = array();
		
		if(!$size) 
		{
			$errors[] = 'There was a problem loading image ' . $img_name . '.' . $img_ext . ' in mod_authorlist';
		
		} else {
							
			$sub_folder = $item->image ? '0' . substr($item->id, -1) : 'default';
							
			if ( $img_type ) {
				$img_ext = $img_type;
			}
	
			$origw = $size[0];
			$origh = $size[1];
			if( ($origw<$width && $origh<$height)) {
				$width = $origw;
				$height = $origh;
			}
			
			$prefix = substr($proportion,0,1) . "_".$width."_".$height."_".($proportion=='fill'?$bgcolor."_":"").($item->image?$item->id."_":"");
	
			$thumb_file = $prefix . str_replace(array( JPATH_ROOT, ':', '/', '\\', '?', '&', '%20', ' '),  '_' ,urlencode($img_name) . '.' . $img_ext);		
			
			//$thumb_path = __DIR__ . '/thumbs/' . $sub_folder . '/' . $thumb_file;
			$thumb_path = dirname(__FILE__).'/thumbs/' . $sub_folder . '/' . $thumb_file;
			
			$attribs = array();
			
			if(file_exists($thumb_path))	{
				$size = @getimagesize($thumb_path);
				if($size) {
					$attribs['width']  = $size[0];
					$attribs['height'] = $size[1];
				}
			} else {
		
				modAuthorListHelper::calculateSize($origw, $origh, $width, $height, $proportion, $newwidth, $newheight, $dst_x, $dst_y, $src_x, $src_y, $src_w, $src_h);
	
				switch(strtolower($size['mime'])) {
					case 'image/png':
						$imagecreatefrom = "imagecreatefrompng";
						break;
					case 'image/gif':
						$imagecreatefrom = "imagecreatefromgif";
						break;
					case 'image/jpeg':
						$imagecreatefrom = "imagecreatefromjpeg";
						break;
					default:
						$errors[] = "Unsupported image type $img_name.$img_ext ".$size['mime'];
				}
	
				
				if ( !function_exists ( $imagecreatefrom ) ) {
					$errors[] = "Failed to process $img_name.$img_ext in mod_authorlist. Function $imagecreatefrom doesn't exist.";
				}
				
				$src_img = $imagecreatefrom($img_path);
				
				if (!$src_img) {
					$errors[] = "There was a problem to process image $img_name.$img_ext ".$size['mime'] . ' in mod_authorlist';
				}
				
				$dst_img = ImageCreateTrueColor($width, $height);
				
				// $bgcolor = imagecolorallocatealpha($image, 200, 200, 200, 127);
				
				imagefill( $dst_img, 0,0, $bgcolor);
				if ( $proportion == 'transparent' ) {
					imagecolortransparent($dst_img, $bgcolor);
				}
				
				imagecopyresampled($dst_img,$src_img, $dst_x, $dst_y, $src_x, $src_y, $newwidth, $newheight, $src_w, $src_h);		
				
				switch(strtolower($img_ext)) {
					case 'png':
						$imagefunction = "imagepng";
						break;
					case 'gif':
						$imagefunction = "imagegif";
						break;
					default:
						$imagefunction = "imagejpeg";
				}
				
				if($imagefunction=='imagejpeg') {
					$result = @$imagefunction($dst_img, $thumb_path, 80 );
				} else {
					$result = @$imagefunction($dst_img, $thumb_path);
				}
	


				imagedestroy($src_img);
				if(!$result) {				
					if(!isset($disablepermissionwarning)) {
					$errors[] = 'Could not create image:<br />' . $thumb_path . ' in mod_authorlist.<br /> Check if the folder exists and if you have write permissions:<br /> ' . dirname(__FILE__) . '/thumbs/' . $sub_folder;
					}
					$disablepermissionwarning = true;
				} else {
					imagedestroy($dst_img);
				}
			}
		}
		
		if (count($errors)) {
			JError::raiseWarning(404, implode("\n", $errors));
			return false;
		}
				
		$image = $img_base."modules/mod_authorlist/thumbs/$sub_folder/" . urlencode(basename($thumb_path));
		
		return  JHTML::_('image', $image, $item->name, $attribs);
    }
	
	public static function calculateSize($origw, $origh, &$width, &$height, &$proportion, &$newwidth, &$newheight, &$dst_x, &$dst_y, &$src_x, &$src_y, &$src_w, &$src_h) {
		
		if(!$width ) {
			$width = $origw;
		}

		if(!$height ) {
			$height = $origh;
		}

		if ( $height > $origh ) {
			$newheight = $origh;
			$height = $origh;
		} else {
			$newheight = $height;
		}
		
		if ( $width > $origw ) {
			$newwidth = $origw;
			$width = $origw;
		} else {
			$newwidth = $width;
		}
		
		$dst_x = $dst_y = $src_x = $src_y = 0;

		switch($proportion) {
			case 'fill':
			case 'transparent':
				$xscale=$origw/$width;
				$yscale=$origh/$height;

				if ($yscale<$xscale){
					$newheight =  round($origh/$origw*$width);
					$dst_y = round(($height - $newheight)/2);
				} else {
					$newwidth = round($origw/$origh*$height);
					$dst_x = round(($width - $newwidth)/2);

				}

				$src_w = $origw;
				$src_h = $origh;
				break;

			case 'crop':

				$ratio_orig = $origw/$origh;
				$ratio = $width/$height;
				if ( $ratio > $ratio_orig) {
					$newheight = round($width/$ratio_orig);
					$newwidth = $width;
				} else {
					$newwidth = round($height*$ratio_orig);
					$newheight = $height;
				}
					
				$src_x = ($newwidth-$width)/2;
				$src_y = ($newheight-$height)/2;
				$src_w = $origw;
				$src_h = $origh;				
				break;
				
 			case 'only_cut':
				// }
				$src_x = round(($origw-$newwidth)/2);
				$src_y = round(($origh-$newheight)/2);
				$src_w = $newwidth;
				$src_h = $newheight;
				
				break; 
				
			case 'bestfit':
				$xscale=$origw/$width;
				$yscale=$origh/$height;

				if ($yscale<$xscale){
					$newheight = $height = round($width / ($origw / $origh));
				}
				else {
					$newwidth = $width = round($height * ($origw / $origh));
				}
				$src_w = $origw;
				$src_h = $origh;	
				
				break;
			}

	}
}
